﻿using Infragistics.Controls.Menus;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;

namespace XamDataTreeBug
{
    /// <summary>
    /// Interaktionslogik für MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window, INotifyPropertyChanged
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
        }

        private ObservableCollection<SimpleViewModel> mDataItems;

        public ObservableCollection<SimpleViewModel> DataItems
        {
            get
            {
                return mDataItems;
            }
            set
            {
                mDataItems = value;
                PropertyChanged(this, new PropertyChangedEventArgs("DataItems"));
            }
        }

        private void StackPanel_PreviewMouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            mMouseDownPos = e.GetPosition(null);

        }

        private void StackPanel_MouseMove(object sender, MouseEventArgs e)
        {
            Point mousePos = e.GetPosition(null);
            Vector diff = mMouseDownPos - mousePos;

            if ((e.LeftButton == MouseButtonState.Pressed) &&
                ((Math.Abs(diff.X) > SystemParameters.MinimumHorizontalDragDistance) ||
                (Math.Abs(diff.Y) > SystemParameters.MinimumVerticalDragDistance)))
            {
                object[] items = null;
                //items = new object[] {treeView.SelectedItem};
                //DependencyObject dragSource = treeView;
                items = Tree.SelectedDataItems;
                DependencyObject dragSource = Tree.ActiveNode.Control;
                //                if ((Tree.SelectedDataItems != null) && (Tree.SelectedDataItems.Length > 0))
                if (items != null)
                {
                    DragDrop.DoDragDrop(dragSource, new DataObject("ViewModels", items.ToList()),
                        DragDropEffects.All);
                }
            }

        }

        private void StackPanel_DragEnter(object sender, DragEventArgs e)
        {
            SetAllowed(e);
            e.Handled = true;
        }

        private void StackPanel_DragLeave(object sender, DragEventArgs e)
        {
            e.Effects = DragDropEffects.None;
            e.Handled = true;
        }

        private void StackPanel_DragOver(object sender, DragEventArgs e)
        {
            SetAllowed(e);
            e.Handled = true;
        }

        private void SetAllowed(DragEventArgs e)
        {
            BindingExpression expr = (e.Source as TextBlock).GetBindingExpression(TextBlock.TextProperty);
            SimpleViewModel vm = (expr.DataItem as XamDataTreeNodeDataContext).Data as SimpleViewModel;
            List<object> list =
                (from item in (List<object>) e.Data.GetData("ViewModels") where item is SimpleViewModel select item).ToList();
            e.Effects = (list.IndexOf(vm) >= 0) ? DragDropEffects.None : DragDropEffects.Move;
        }

        private void StackPanel_Drop(object sender, DragEventArgs e)
        {
            BindingExpression expr = (e.Source as TextBlock).GetBindingExpression(TextBlock.TextProperty);
            SimpleViewModel target = (expr.DataItem as XamDataTreeNodeDataContext).Data as SimpleViewModel;
            List<object> list =
                (from item in (List<object>)e.Data.GetData("ViewModels") where item is SimpleViewModel select item).ToList();
            SimpleViewModel source = list[0] as SimpleViewModel;
            source.IsSelected = false;
            source.Parent.Children.Remove(source);
            Dispatcher.BeginInvoke((Action)(() =>
            {
                target.Children.Add(source);
                source.Parent = target;
                source.IsSelected = false;
            }));
        }



        public event PropertyChangedEventHandler PropertyChanged;
        private Point mMouseDownPos;

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            DataContext = this;
            ObservableCollection<SimpleViewModel> datas = new ObservableCollection<SimpleViewModel>();
            SimpleViewModel top = new SimpleViewModel() { Children = new ObservableCollection<SimpleViewModel>(), Text = "Top" };
            SimpleViewModel child1 = new SimpleViewModel
            {
                Children = new ObservableCollection<SimpleViewModel>(),
                Parent = top,
                Text = "Child1"
            };
            top.Children.Add(child1);
            SimpleViewModel child2 = new SimpleViewModel
            {
                Children = new ObservableCollection<SimpleViewModel>(),
                Parent = top,
                Text = "Child2"
            };
            top.Children.Add(child2);

            child1.Children.Add(new SimpleViewModel()
            {
                Children = new ObservableCollection<SimpleViewModel>(),
                Parent = child1,
                Text = "Grandchild"
            });
            datas.Add(top);
            DataItems = datas;

        }


    }
}
